using System;
using System.Data;
using System.Data.SqlClient;
using System.Configuration;

namespace DayOfDotNet
{
	public class BooksData
	{
		// Returns a DataView containing all the basic book info.
		public DataView GetAllBooks()
		{
			SqlConnection con = new SqlConnection(ConfigurationSettings.AppSettings["DotNetBooksConString"]);
			SqlDataAdapter sda = new SqlDataAdapter("GetAllBooks", con);
			sda.SelectCommand.CommandType = CommandType.StoredProcedure;
			DataSet ds = new DataSet();
			sda.Fill(ds, "Books");
			return ds.Tables["Books"].DefaultView;
		}

		// Returns a DataSet containing two DataTables
		//- "Books" and "Authors"
		//- for a specific book, using the ISBN number.
		public DataSet GetBookInfo(string isbn)
		{
			SqlConnection con = new SqlConnection(ConfigurationSettings.AppSettings["DotNetBooksConString"]);
			SqlDataAdapter sda = new SqlDataAdapter("GetBookInfo", con);
			sda.SelectCommand.CommandType = CommandType.StoredProcedure;
			sda.SelectCommand.Parameters.Add(new SqlParameter("@isbn", SqlDbType.VarChar, 14)).Value = isbn;
			DataSet ds = new DataSet();
			sda.Fill(ds);
			ds.Tables[0].TableName="Books";
			ds.Tables[1].TableName="Authors";
			return ds;
		}
		
		// Returns a DataSet containing two DataTables
		//- "Table" (Books) and "Table1" (Authors) 
		public DataSet GetAllBookInfoDataSet()
		{
			SqlConnection con = new SqlConnection(ConfigurationSettings.AppSettings["DotNetBooksConString"]);
			SqlDataAdapter sda = new SqlDataAdapter("GetAllBooks", con);
			sda.SelectCommand.CommandType = CommandType.StoredProcedure;
			DataSet ds = new DataSet();
			sda.Fill(ds, "Books");
			sda.SelectCommand.CommandText = "GetAllAuthorsByBooks";
			sda.Fill(ds, "Authors");
			
			return ds;
		}
		
		
		public DataView GetPublishers()
		{
			SqlConnection con = new SqlConnection(ConfigurationSettings.AppSettings["DotNetBooksConString"]);
			SqlDataAdapter sda = new SqlDataAdapter("GetAllPublishers", con);
			sda.SelectCommand.CommandType = CommandType.StoredProcedure;
			DataSet ds = new DataSet();
			sda.Fill(ds, "Publishers");
			return ds.Tables["Publishers"].DefaultView;
		}
		
		public void UpdateBook(
			string isbn,
			string title,
			string publisher,
			DateTime releaseDate,
			string price,
			string description,
			string amazonLink
		)
		{
			SqlConnection con = new SqlConnection(ConfigurationSettings.AppSettings["DotNetBooksConString"]);
			SqlCommand cmd = new SqlCommand("UpdateBook", con);
			cmd.CommandType = CommandType.StoredProcedure;
			
			cmd.Parameters.Add(new SqlParameter("@isbn", SqlDbType.VarChar,14)).Value = isbn;
			cmd.Parameters.Add(new SqlParameter("@title", SqlDbType.VarChar,200)).Value = title;
			cmd.Parameters.Add(new SqlParameter("@publisher", SqlDbType.Int)).Value = publisher;
			cmd.Parameters.Add(new SqlParameter("@releaseDate", SqlDbType.DateTime)).Value = releaseDate;
			cmd.Parameters.Add(new SqlParameter("@price", SqlDbType.Money)).Value = Decimal.Parse(price);
			cmd.Parameters.Add(new SqlParameter("@description", SqlDbType.NText)).Value = description;
			cmd.Parameters.Add(new SqlParameter("@amazonLink", SqlDbType.VarChar,255)).Value = amazonLink;
			
			con.Open();
			cmd.ExecuteNonQuery();
			con.Close();		
		}
	}
}
