using System;
using System.Data;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;

namespace DayOfDotNet
{
	public class EditableDataGrid : System.Web.UI.Page
	{
		//Map a variable for the DataGrid server control
		protected DataGrid BookDataGrid;
		//Declare a class-level DataView for editing
		private DataView _publisherList;
			
		private void Page_Load(object sender, System.EventArgs e)
		{
			if(!Page.IsPostBack)
			{
				BindData();
			}
		}
		
		private void BindData()
		{			
			//When the page is loaded, create the BookDataSet.
			//Create a BooksData class instance, 
			//and invoke GetAllBooks()
			BooksData _booksData = new BooksData();
			DataView BookDataView = _booksData.GetAllBooks();
			
			//The BookDataGrid is the first table in the DataSet.
			BookDataGrid.DataSource = BookDataView;
			BookDataGrid.DataBind();		
		}
		
		protected void DataGrid_OnEditCommand(object sender, DataGridCommandEventArgs e)
		{
			//Create the _publisherList DataView
			BooksData _booksData = new BooksData();
			_publisherList = _booksData.GetPublishers();
			
			//Set the EditItemIndex
			((DataGrid)sender).EditItemIndex = e.Item.ItemIndex;
			BindData();
		}
		
		protected void DataGrid_OnCancelCommand(object sender, DataGridCommandEventArgs e)
		{
			//Turn off editing by reseting the EditItemIndex
			((DataGrid)sender).EditItemIndex = -1;
			BindData();

		}
		
		protected void DataGrid_OnUpdateCommand(object sender, DataGridCommandEventArgs e)
		{
			//The ISBN is the Text of the second column
			string isbn = e.Item.Cells[1].Text;
			
			//Capture the Title TextBox
			TextBox Title = (TextBox)e.Item.Cells[2].Controls[0];
			
			//Capture the EditItemTemplate controls
			DropDownList Publisher = (DropDownList)e.Item.Cells[3].FindControl("Publishers");
			Calendar ReleaseDate = (Calendar)e.Item.Cells[3].FindControl("ReleaseDate");
			TextBox Price = (TextBox)e.Item.Cells[3].FindControl("Price");
			TextBox Description = (TextBox)e.Item.Cells[3].FindControl("Description");
			TextBox AmazonLink = (TextBox)e.Item.Cells[3].FindControl("AmazonLink");
			
			//Update the database with the user values from the server controls
			BooksData _booksData = new BooksData();
			_booksData.UpdateBook(
				isbn,
				Title.Text.Trim(),
				Publisher.SelectedItem.Value,
				ReleaseDate.SelectedDate,
				Price.Text.Trim(),
				Description.Text.Trim(),
				AmazonLink.Text.Trim()
			);
		
			//Turn off editing by reseting the EditItemIndex
			((DataGrid)sender).EditItemIndex = -1;
			//Retreive the data and bind the grid
			BindData();
		}
		
		protected DataView GetPublisherList()
		{
			//_publisherList is a class-level DataView
			return _publisherList;
		}

		protected int GetSelectedPublisher(string _publisher)
		{
			//_publisherList is a class-level DataView
			for(int i=0; i < _publisherList.Count; i++)
			{
				//Return the selected publisher value
				//when the PublisherName matches
				if(_publisherList[i]["Name"].ToString() == _publisher)
				{
					return i;
				}
			}
			
			//If there is no match, return 0
			return 0;
		}
	}
}