Imports System
Imports System.Data
Imports System.Drawing
Imports System.Web
Imports System.Web.UI
Imports System.Web.UI.WebControls
Imports System.Web.UI.HtmlControls

Namespace DayOfDotNet

	Public Class DynamicEmbeddedDataGrid
		Inherits System.Web.UI.Page

		Protected BookDataGrid As DataGrid
		Private BookDataSet As DataSet

		Private Sub Page_Load(sender As object, e As System.EventArgs)
			'Set a default ISBN
			Dim isbn As String = "0-672-32106-8"
			
			'When the page is loaded, create the BookDataSet.
			'Create a BooksData class instance
			Dim _booksData As BooksData= New BooksData()
			
			'If there is an ISBN in the QueryString, use it instead.
			'and invoke GetBookInfo() passing in the ISBN string.
			If Not HttpContext.Current.Request.QueryString("isbn") = Nothing Then
				isbn = HttpContext.Current.Request.QueryString("isbn")
				BookDataSet = _booksData.GetBookInfo(isbn)
			Else
				BookDataSet = _booksData.GetAllBookInfoDataSet()
			End If

			BookDataGrid.DataSource = BookDataSet.Tables("Books")
			BookDataGrid.DataBind()
		End Sub

		'Use the OnItemCreated event handler to dynamically add an embedded DataGrid
		Protected Sub BookDataGrid_OnItemDataBound(sender As Object, e As DataGridItemEventArgs)
			'When each row is created in the DataGrid, eval the ItemType
			If e.Item.ItemType = ListItemType.Item XOr e.Item.ItemType = ListItemType.AlternatingItem Then
				'If the ItemType is Item or AlternatingItem, 
				'Create a New DataGrid object named AuthorsDataGrid
				Dim AuthorsDataGrid As DataGrid = New DataGrid()
				
				'Format the DataGrid to look cool.
				AuthorsDataGrid.ItemStyle.Font.Name = "Verdana"
				AuthorsDataGrid.ItemStyle.Font.Size = FontUnit.XSmall
				AuthorsDataGrid.BorderWidth = Unit.Pixel(0)
				AuthorsDataGrid.ShowHeader = False
				AuthorsDataGrid.AlternatingItemStyle.BackColor = Color.FromName("Tan")
				
				'Do not autogenerate columns.
				AuthorsDataGrid.AutoGenerateColumns = False
				
				'For the only column in the AuthorsDataGrid, create a HyperLinkColumn.
				Dim hlc As HyperLinkColumn = New HyperLinkColumn()
				
				'Format the link to send a mail message.
				hlc.DataNavigateUrlField = "Email"
				hlc.DataNavigateUrlFormatString = "mailto:{0}"
				hlc.DataTextField = "AuthorName"
				hlc.ItemStyle.Wrap = False
				
				'Add the HyperlinkColumn to the AuthorsDataGrid.
				AuthorsDataGrid.Columns.Add(hlc)
				
				'Get the Authors DataView and filter it for the current ISBN
				Dim _authors As DataView = BookDataSet.Tables("Authors").DefaultView
				_authors.RowFilter = "ISBN='" & e.Item.Cells(0).Text & "'"
				
				'Bind the DataGrid.
				AuthorsDataGrid.DataSource = _authors
				AuthorsDataGrid.DataBind()
				
				'Add the AuthorsDataGrid to the BooksDataGrid.
				e.Item.Cells(3).Controls.Add(AuthorsDataGrid)
			End If
		End Sub
	End Class
End Namespace