using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;

namespace CustomControls.Ex10
{
	public class Ex10_Control : Control, INamingContainer
	{
		public event Ex10EventHandler ItemCommand;
		public event Ex10EventHandler IndexChanged;
	
		protected override void CreateChildControls()
		{
			Controls.Add(new LiteralControl("<h3>FirstChild Control</h3>"));
			Ex10_Control_Child cc1 = new Ex10_Control_Child("FirstChild", 5);
			Controls.Add(cc1);
			Controls.Add(new LiteralControl("<br><br>"));

			Controls.Add(new LiteralControl("<h3>SecondChild Control</h3>"));
			Ex10_Control_Child cc2 = new Ex10_Control_Child("SecondChild", 5);
			Controls.Add(cc2);
		}

		protected override bool OnBubbleEvent(object source, System.EventArgs args)
		{
			Ex10EventArgs e = args as Ex10EventArgs;
			if (e != null)	{
				if (e.CommandName == "IndexChanged" 
					&& ItemCommand != null)
					IndexChanged(this, e);
				else if (IndexChanged != null)
					ItemCommand(this, e);
				return true;
			}
			return false;
		}
	}

	public class Ex10_Control_Child : Control, INamingContainer
	{
		private int count;
		private string childName;

		public string ChildName	{
			get {return childName;}
		}

		public Ex10_Control_Child(string childName, int count)
		{
			this.count = count;
			this.childName = childName;
		}

		protected override void CreateChildControls()
		{
			for (int i=1;i<=count;i++) {
				Button btn = new Button();
				btn.Text = childName + " " + i;
				btn.CommandName = "CommandOf" + childName;
				btn.CommandArgument = i + "th";
				Controls.Add(btn);

				DropDownList ddl1 = new DropDownList();
				for (int j=0;j<20;j++)
					ddl1.Items.Add(j.ToString());
				ddl1.ID = "DDL" + i;
				ddl1.SelectedIndex = 0;
				ddl1.AutoPostBack = true;
				ddl1.SelectedIndexChanged += new EventHandler(DDL_SelectedIndexChanged);
				Controls.Add(ddl1);
			}
		}

		protected override bool OnBubbleEvent(object source, EventArgs e) 
		{
			if (e is CommandEventArgs) 
			{
				Ex10EventArgs args =
					new Ex10EventArgs(this, source, (CommandEventArgs)e);

				RaiseBubbleEvent(this, args);
				//̺Ʈ ֻ Ʈѿ óϵ Ѵ.
				return true;
			}
			return false;
		}

		private void DDL_SelectedIndexChanged(Object sender, EventArgs e) 
		{
			Ex10EventArgs args =
				new Ex10EventArgs(this, sender, new CommandEventArgs("IndexChanged", ""));

			RaiseBubbleEvent(this, args);
		}
	}

	//̺Ʈ    Ʈѷ Ѱ .
	public sealed class Ex10EventArgs : CommandEventArgs 
	{
		private Ex10_Control_Child item;
		private object commandSource;

		public Ex10EventArgs (Ex10_Control_Child item, object commandSource, CommandEventArgs originalArgs) :
			base(originalArgs) 
		{
			this.item = item;
			this.commandSource = commandSource;
		}

		public Ex10_Control_Child Item {
			get {return item;}
		}

		public object CommandSource {
			get {return commandSource;}
		}
	}

	public delegate void Ex10EventHandler(object source, Ex10EventArgs  e);
}