using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Collections;

namespace CustomControls.TemplatedControls
{
	[ParseChildren(true)]
	public class TemplatedControl4 : WebControl, INamingContainer
	{
		private ITemplate itemTemplate = null;
		private ITemplate alternatingTemplate = null;
		private IEnumerable dataSource = null;

		public IEnumerable DataSource {
			get {return dataSource;}
			set {dataSource = value;}
		}

		[PersistenceMode(PersistenceMode.InnerProperty),
		TemplateContainer(typeof(TemplateItem))]
		public ITemplate ItemTemplate {
			get {return itemTemplate;}
			set {itemTemplate = value;}
		}

		[PersistenceMode(PersistenceMode.InnerProperty),
		TemplateContainer(typeof(TemplateItem))]
		public ITemplate AlternatingTemplate {
			get {return alternatingTemplate;}
			set {alternatingTemplate = value;}
		}

		private int ItemCount {
			get {
				object o = ViewState["itemCount"];
				return (o == null) ? 0 : (int)o;
			}
			set {ViewState["itemCount"] = value;}
		}
		
		protected override void OnDataBinding(System.EventArgs e)
		{
			base.OnDataBinding(e);

			Controls.Clear();

			if (dataSource != null) {
				if(HasChildViewState)
					ClearChildViewState();

				int count = 0;
				foreach(object dataItem in dataSource) {
					TemplateItem item = new TemplateItem(dataItem);
					if (count % 2 == 0 || alternatingTemplate == null)
						ItemTemplate.InstantiateIn(item);
					else
						alternatingTemplate.InstantiateIn(item);

					//item.DataBind();
					Controls.Add(item);
					count++;
				}

				ChildControlsCreated = true;
				ItemCount = count;
			}
		}

		protected override void CreateChildControls() 
		{
			Controls.Clear();

			if (ItemCount != 0) {
				for (int i=0;i<ItemCount;i++) {
					TemplateItem item = new TemplateItem(null);
					if (i % 2 == 0 || alternatingTemplate == null)
						ItemTemplate.InstantiateIn(item);
					else
						alternatingTemplate.InstantiateIn(item);

					Controls.Add(item);
				}
			}
		}

		public class TemplateItem : Control, INamingContainer 
		{
			private object dataItem;

			public TemplateItem(object datItm) 
			{
				dataItem = datItm;
			}

			public object DataItem {
				get {return dataItem;}
				set {dataItem = value;}
			}
		}
	}
}
