using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;

namespace CustomControls.TextBox
{
	[DefaultProperty("Text"),
	ValidationProperty("Text"),
	ParseChildren(false),
	ToolboxData("<{0}:SimpleTextBox runat=server></{0}:SimpleTextBox>")]
	public class SimpleTextBox : WebControl, IPostBackDataHandler
	{
		public event EventHandler TextChanged;

		private bool isRunTime;

		#region Ӽ 
		private string text;
		private bool autoPostBack;
		private int columns = 0;
		private int rows = 0;
		private TextBoxMode textMode = TextBoxMode.SingleLine;
		#endregion
		#region Ӽ
		[Bindable(true), 
		Category("Appearance"), 
		DefaultValue("")] 
		public string Text {
			get	{return text;}
			set	{text = value;}
		}

		[Bindable(true), 
		Category("Behavior"), 
		DefaultValue(false)]
		public bool AutoPostBack {
			get	{return autoPostBack;}
			set	{autoPostBack = value;}
		}

		[Bindable(true), 
		Category("Appearance"), 
		DefaultValue(0)]
		public int Columns {
			get	{return columns;}
			set	{columns = value;}
		}

		[Bindable(true), 
		Category("Appearance"), 
		DefaultValue(0)]
		public int Rows	{
			get	{return rows;}
			set	{rows = value;}
		}

		[Bindable(true), 
		Category("Appearance"), 
		DefaultValue(TextBoxMode.SingleLine)]
		public TextBoxMode TextMode	{
			get	{return textMode;}
			set	{textMode = value;}
		}
		#endregion
		#region ޼ҵ

		//public SimpleTextBox() : base(HtmlTextWriterTag.Input) {}

		protected override void AddParsedSubObject(object obj)
		{
			if (obj is LiteralControl)
				Text = ((LiteralControl)obj).Text;
		}

		public bool LoadPostData(string postDataKey, 
			System.Collections.Specialized.NameValueCollection postCollection)
		{
			string newVal = postCollection[postDataKey];

			if(text != newVal) {
				text = newVal;
				return true;
			}

			return false;
		}

		public void RaisePostDataChangedEvent()
		{
			if (TextChanged != null)
				TextChanged(this, EventArgs.Empty);
		}

		protected override void AddAttributesToRender(HtmlTextWriter writer)
		{
			writer.AddAttribute(HtmlTextWriterAttribute.Name, UniqueID);
			switch (textMode) {
				case TextBoxMode.SingleLine :
					writer.AddAttribute(HtmlTextWriterAttribute.Type, "input");
					writer.AddAttribute(HtmlTextWriterAttribute.Value, text);
					break;
				case TextBoxMode.Password :
					writer.AddAttribute(HtmlTextWriterAttribute.Type, "password");
					writer.AddAttribute(HtmlTextWriterAttribute.Value, text);
					break;
				case TextBoxMode.MultiLine :
					if (rows > 0)
						writer.AddAttribute(HtmlTextWriterAttribute.Rows, rows.ToString());
					break;
			}
			if (columns > 0)
				writer.AddAttribute(HtmlTextWriterAttribute.Cols, columns.ToString());

			// Ÿӿ  ʵѴ.
			// Ÿӿ Ⱑ Ǹ Request ü 
			//    Ʈ  ̻ϰ δ.
			if (isRunTime) {
				//W3C XML Doom  1̻̰ ECMAScript  1.2 ̻
				bool scriptEnable = Page.Request.Browser.W3CDomVersion.Major >= 1 && 
					Page.Request.Browser.EcmaScriptVersion.CompareTo(new Version(1, 2)) >= 0;
				if (autoPostBack && scriptEnable)
					writer.AddAttribute(HtmlTextWriterAttribute.Onchange, Page.GetPostBackClientEvent(this, ""));
			}
		}
		
		protected override void RenderContents(HtmlTextWriter writer)
		{
			if (textMode == TextBoxMode.MultiLine)
				writer.Write(text);
		}

		public override void RenderBeginTag(HtmlTextWriter writer)
		{
			/**************************************************
			 * RenderBeginTag ޼ҵ带    
			 * base.RenderBeginTag ȣ 
			 * AddAttributesToRender ޼尡 ȣ ʴ´.
			 * ׷ AddAttributesToRender  ȣѴ.
			***************************************************/
			AddAttributesToRender(writer);

			switch (textMode) {
				case TextBoxMode.SingleLine :
					writer.RenderBeginTag(HtmlTextWriterTag.Input);
					break;
				case TextBoxMode.Password :
					writer.RenderBeginTag(HtmlTextWriterTag.Input);
					break;
				case TextBoxMode.MultiLine :
					writer.RenderBeginTag(HtmlTextWriterTag.Textarea);
					break;
			}
		}

		public override void RenderEndTag(HtmlTextWriter writer)
		{
			writer.RenderEndTag();
		}

		protected override void OnPreRender(System.EventArgs e)
		{
			// Ÿӿ OnPreRender ȣ ʴ´.
			isRunTime = true;
		}

		protected override void LoadViewState(object savedState)
		{
			if (savedState != null)	{
				object[] stateArr = (object[])savedState;
				base.LoadViewState(stateArr[0]);
				if (stateArr[1] != null)
					text = stateArr[1].ToString();
				autoPostBack = (bool)stateArr[2];
				columns = (int)stateArr[3];
				rows = (int)stateArr[4];
				textMode = (TextBoxMode)stateArr[5];
			}
		}

		protected override object SaveViewState()
		{
			object[] stateArr = new object[6];
			stateArr[0] = base.SaveViewState();
			stateArr[1] = text;
			stateArr[2] = autoPostBack;
			stateArr[3] = columns;
			stateArr[4] = rows;
			stateArr[5] = textMode;

			return (object)stateArr;
		}
		#endregion
	}
}