using System;
using System.Drawing;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;

namespace CustomControls.Validators
{
	[DefaultProperty("ErrorMessage")]
	public abstract class SimpleBaseValidator2 : System.Web.UI.WebControls.Label, IValidator
	{
		private const string ValidatorFileName = "SimpleValidation.js";
		private const string ValidatorIncludeScriptKey = "SimpleValidatorIncludeScript";
		private const string ValidatorStartupScript = @"
<script language=""javascript"">
<!--
var Page_ValidationActive = false;
if (typeof(Page_SimpleValidationVer) == ""undefined"")
    alert(""{0}"");
else
    ValidatorOnLoad();

function ValidatorOnSubmit() {{
    if (Page_ValidationActive) {{
        return ValidatorCommonOnSubmit();
    }}
}}
// -->
</script>
        ";
		private const string IncludeScriptFormat = @"
<script language=""javascript"" src=""{0}{1}""></script>";

		private bool renderUplevel;
		private bool isValid;


		[DefaultValue(typeof(Color), "Red")]                                         
		public override Color ForeColor {
			get {return base.ForeColor;}
			set {base.ForeColor = value;}
		}

		[Category("Behavior"),
		DefaultValue(""),
		TypeConverter(typeof(ValidatedControlConverter))]                                         
		public string ControlToValidate {
			get { 
				object o = ViewState["ControlToValidate"];
				return((o == null) ? String.Empty : (string)o);
			}
			set {ViewState["ControlToValidate"] = value;}
		}

		[Category("Appearance"),
		DefaultValue("")]
		public string ErrorMessage {
			get {return base.Text;}
			set {base.Text = value;}
		}

		[Browsable(false),
		DefaultValue(true),
		DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public bool IsValid 
		{
			get {return isValid;}
			set {isValid = value;}
		}

		[Category("Behavior"),
		DefaultValue(true)]
		public bool MessageDisplay {
			get {
				object o = ViewState["MessageDisplay"];
				return((o == null) ? true : (bool)o);
			}
			set {ViewState["MessageDisplay"] = value;}
		}

		[Category("Behavior"),
		DefaultValue(true)]
		public bool EnableClientScript {
			get {
				object o = ViewState["EnableClientScript"];
				return((o == null) ? true : (bool)o);
			}
			set {ViewState["EnableClientScript"] = value;}
		}

		//Ŭ̾Ʈ ũƮ   
		protected bool RenderUplevel {
			get {return renderUplevel;}
		}

		protected string GetControlValidationValue(string name) 
		{
			/*****************************************
			 * ȿ ˻縦  Ʈ ˻簪
			 *  ޼ҵ̴.
			 * ˻縦  Ʈ ̸  
			 * Ʈ ˻簪 ; ϱ 
			 * ÷ Ѵ.
			 *****************************************/
			// ˻ Ʈ ´.
			Control c = NamingContainer.FindControl(name);            
			if (c == null) 
				return null;

			// Ʈѿ ȿ ˻縦  Ӽ PropertyDescriptor ´.
			PropertyDescriptor prop = GetValidationProperty(c);
			if (prop == null) 
				return null;

			// Ӽ  ´.
			object value = prop.GetValue(c);
			if (value != null) 
				return value.ToString();
			else 
				return string.Empty;
		}                

		//־ Ʈ ValidationPropertyAttribute Ʈ  
		//Ӽ ִٸ  Ӽ  PropertyDescriptor ü ȯѴ.
		public static PropertyDescriptor GetValidationProperty(object component) 
		{
			//component ü  ƮƮ ߿ 
			//ValidationPropertyAttribute   ´.
			//ValidationPropertyAttribute   
			//Ӽ ˻簪  ִ.
			ValidationPropertyAttribute valProp = (ValidationPropertyAttribute)TypeDescriptor.GetAttributes(component)[typeof(ValidationPropertyAttribute)];
			if (valProp != null && valProp.Name != null) 
				//valProp.Name  ̸  Ӽ 
				//PropertyDescriptor ü ȯ
				return TypeDescriptor.GetProperties(component, null)[valProp.Name];

			return null;
		}

		protected override void OnInit(EventArgs e) 
		{
			base.OnInit(e);

			isValid = true;
			// ޼ 
			ForeColor = Color.Red;
			//Page Validators Ӽ  ͸ ߰
			Page.Validators.Add(this);
		}        

		protected override void OnUnload(EventArgs e) 
		{
			if (Page != null) 
				Page.Validators.Remove(this);

			base.OnUnload(e);
		}

		protected abstract bool EvaluateIsValid();

		public void Validate() 
		{
			if (!Visible) 
			{
				IsValid = true;
				return;
			}

			Control parent = Parent;
			while (parent != null) 
			{
				if (!parent.Visible) 
				{
					IsValid = true;
					return;
				}
				parent = parent.Parent;
			}

			IsValid = EvaluateIsValid();
		}

		//־ ID  Ʈ ClientID Ѵ.
		protected string GetControlRenderID(string name) 
		{
			Control c = FindControl(name);            
			if (c == null) 
				return String.Empty;

			return c.ClientID;
		}

		protected override void AddAttributesToRender(HtmlTextWriter writer) 
		{
			base.AddAttributesToRender(writer);

			//⿡ ߰Ǵ Ӽ Ŭ̾Ʈ 
			//ũƮ Ѵ.
			if (renderUplevel) 
			{
				if (ID == null) 
					writer.AddAttribute("id", ClientID);

				if (ControlToValidate.Length > 0) {
					string controlRenderID = GetControlRenderID(ControlToValidate);
					if (controlRenderID == String.Empty)
						throw new Exception("ControlToValidate  Ʈ ã  ϴ.");

					writer.AddAttribute("controltovalidate", controlRenderID);
				}

				if (ErrorMessage.Length > 0) 
					writer.AddAttribute("errormessage", ErrorMessage, true);

				if (!IsValid) 
					writer.AddAttribute("isvalid", "False");

				writer.AddAttribute("messagedisplay", MessageDisplay.ToString().ToLower());
				writer.AddStyleAttribute("visibility","hidden");
			}
		}

		protected override void OnPreRender(EventArgs e) 
		{
			base.OnPreRender(e);

			renderUplevel = DetermineRenderUplevel();

			// û  Ŭ̾Ʈ ũƮ 
			//  ̸ Ŭ̾Ʈ ũƮ 
			if (renderUplevel) {
				RegisterValidatorCommonScript();
				RegisterValidatorDeclaration();
			}
		}

		protected virtual bool DetermineRenderUplevel() 
		{
			// ü Page äȿ ,  
			// Ե ʾҰų  Ÿӿ false ȯ
			Page page = Page;
			if (page == null || page.Request == null) 
				return false;

			//Ŭ̾Ʈ ũƮ    ˻
			return (EnableClientScript 
				&& page.Request.Browser.W3CDomVersion.Major >= 1
				&& page.Request.Browser.EcmaScriptVersion.CompareTo(new Version(1, 2)) >= 0);
		}

		protected void RegisterValidatorCommonScript() 
		{
			string location = null;

			if (!Page.IsClientScriptBlockRegistered(ValidatorIncludeScriptKey)) {
				//location ũƮ ̴.
				// Validator ũƮ /aspnet_client/ Ʒ Ƿ
				//ڽ  ũƮ ⿡ δ° .
				location = Page.Request.ApplicationPath + "/script/";
				location = location.Replace("//","/");

				// Create the client script block.
				string includeScript = String.Format(IncludeScriptFormat, location, ValidatorFileName);
				Page.RegisterClientScriptBlock(ValidatorIncludeScriptKey, includeScript);
			}

			if (!Page.IsStartupScriptRegistered(ValidatorIncludeScriptKey)) {
				if (location == null) 
					location = Page.Request.ApplicationPath + "/script/";

				// ũƮ      ޼.
				string missingScriptMessage = "Validation  O  ϴ. '" + location + ValidatorFileName + "'";

				// Create the startup script block.
				string startupScript = String.Format(ValidatorStartupScript, new object [] {missingScriptMessage, });
				Page.RegisterStartupScript(ValidatorIncludeScriptKey, startupScript);
			}

			Page.RegisterOnSubmitStatement("ValidatorOnSubmit", "return ValidatorOnSubmit();");
		}

		//  ü(span ױ) Ŭ̾Ʈ 
		//  ֵ 迭 Ѵ.
		protected virtual void RegisterValidatorDeclaration() 
		{
			string element = "document.getElementById(\"" + ClientID + "\")";
			Page.RegisterArrayDeclaration("Page_Validators", element);
		}
	}
}
