using System;
using System.Text;
using System.Web.UI;

namespace InitialFocusDemo
{
	/// <summary>
	/// Utility class for a ASP.NET page
	///
	/// Be sure to notice that this code is provided as a technology sample,
	/// 'as is' and no warranties are made by the author.
	///
	/// For questions and comments: Fons.Sonnemans@reflectionit.nl
	///
	/// </summary>
	public class PageUtil
	{

		/// <summary>
		/// Set the InitialFocus to the given control. Only works when JavaScript is supported.
		/// </summary>
		/// <param name="control">Control to set the InitialFocus on.</param>
		public static void SetInitialFocus(Control control) 
		{
			if (control.Page == null) 
			{
				throw new ArgumentException(
				"The Control must be added to a Page before you can set the IntialFocus to it.");
			}
			if (control.Page.Request.Browser.JavaScript == true) 
			{
				// Create JavaScript
				StringBuilder s = new StringBuilder();
				s.Append("\n<SCRIPT LANGUAGE='JavaScript'>\n");
				s.Append("<!--\n");
				s.Append("function SetInitialFocus()\n");
				s.Append("{\n");
				s.Append("   document.");

				// Find the Form
				Control p = control.Parent;
				while (!(p is System.Web.UI.HtmlControls.HtmlForm))
					p = p.Parent;
				s.Append(p.ClientID);

				s.Append("['");
				s.Append(control.UniqueID);
				s.Append("'].focus();\n");
				s.Append("}\n");
				s.Append("// -->\n");
				s.Append("window.onload = SetInitialFocus;\n");
				s.Append("</SCRIPT>");
				
				// Register Client Script
				control.Page.RegisterClientScriptBlock("InitialFocus", s.ToString());
			}
		}
	}
}
